/*
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 * 
 *   http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an
 * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 * KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations
 * under the License.    
 */
package org.fabric3.spi.services.contribution;

import org.fabric3.host.contribution.ContributionException;
import org.fabric3.spi.model.type.ContributionResourceDescription;

/**
 * Interface for services that process contributions. Contribution processing occurs in several phases. Contribution
 * metadata is first processed, after which contained resources are indexed. Indexed {@link Resource}s contain 0..n
 * {@link ResourceElement}s, which are addressable parts. ResourceElements contain a key for a symbol space and a value.
 * When a resource is indexed, only ResourceElement keys are available; their values have not yet been loaded.
 * <p/>
 * The final processing phase is when the contribution is loaded. At this point, all contribution artifacts, including
 * those in depedent contributions, are made available through the provided classloader. Indexed Resources are iterated
 * and all ResourceElement values are loaded via the loader framework. As ResourceElements may refer to other
 * ResourceElements, loading may ocurr recursively.
 *
 * @version $Rev: 2266 $ $Date: 2007-12-12 19:04:29 +0000 (Wed, 12 Dec 2007) $
 */
public interface ContributionProcessor {
    /**
     * Returns the content type this implementation handles.
     *
     * @return the content type this implementation handles
     */
    public abstract String[] getContentTypes();

    /**
     * Processses manifest information for the contribution, including imports and exports.
     *
     * @param contribution The contribution that will be used to hold the results from the processing
     * @throws ContributionException if there was a problem with the contribution
     */
    void processManifest(Contribution contribution) throws ContributionException;

    /**
     * Indexes all contribution resources
     *
     * @param contribution the contribution to index
     * @throws ContributionException if there was a problem indexing
     */
    void index(Contribution contribution) throws ContributionException;

    /**
     * Loads all resources in the contribution.
     *
     * @param contribution The contribution
     * @param loader       the classloader contribution resources must be loaded in
     * @throws ContributionException if there was a problem loading the contribution resoruces
     */
    public void process(Contribution contribution, ClassLoader loader) throws ContributionException;

    /**
     * Updates the resource description for the contribution. Implementations may add required artifact URLs to the
     * description.
     *
     * @param contribution the contribution to update
     * @param description  the description
     * @throws ContributionException if an error occurs updating the description
     */
    public void updateContributionDescription(Contribution contribution, ContributionResourceDescription description)
            throws ContributionException;

}
